@@ -51,6 +51,24 @@ class Settings(object):
         "idle_when_paused":     False,
         "stop_idle":            False,
         "transcode_to_h265":    False,
+        "kb_stop":              "q",
+        "kb_prev":              "<",
+        "kb_next":              ">",
+        "kb_watched":           "w",
+        "kb_unwatched":         "u",
+        "kb_menu":              "c",
+        "kb_menu_esc":          "esc",
+        "kb_menu_ok":           "enter",
+        "kb_menu_left":         "left",
+        "kb_menu_right":        "right",
+        "kb_menu_up":           "up",
+        "kb_menu_down":         "down",
+        "kb_pause":             "space",
+        "kb_debug":             "~",
+        "seek_up":              60,
+        "seek_down":            -60,
+        "seek_right":           5,
+        "seek_left":            -5,
     }
 
     def __getattr__(self, name):
