@@ -4,7 +4,7 @@ import sys
 import urllib.parse
 import time
 
-from threading import RLock
+from threading import RLock, Lock
 from queue import Queue
 
 from . import conffile
@@ -86,6 +86,7 @@ class PlayerManager(object):
         self.evt_queue = Queue()
         self._lock = RLock()
         self._tl_lock = RLock()
+        self._finished_lock = Lock()
         self.last_update = Timer()
         self._jf_settings = None
         self.get_webview = lambda: None
@@ -108,17 +109,25 @@ class PlayerManager(object):
         else:
             log.warning("This mpv version doesn't support on-screen controller.")
 
+        # Wrapper for on_key_press that ignores None.
+        def keypress(key):
+            def wrapper(func):
+                if key is not None:
+                    self._player.on_key_press(key)(func)
+                return func
+            return wrapper
+
         @self._player.on_key_press('CLOSE_WIN')
         @self._player.on_key_press('STOP')
-        @self._player.on_key_press('q')
+        @keypress(settings.kb_stop)
         def handle_stop():
             self.stop()
 
-        @self._player.on_key_press('<')
+        @keypress(settings.kb_prev)
         def handle_prev():
             self.put_task(self.play_prev)
 
-        @self._player.on_key_press('>')
+        @keypress(settings.kb_next)
         def handle_next():
             self.put_task(self.play_next)
 
@@ -140,64 +149,67 @@ class PlayerManager(object):
             else:
                 self.put_task(self.play_next)
 
-        @self._player.on_key_press('w')
+        @keypress(settings.kb_watched)
         def handle_watched():
             self.put_task(self.watched_skip)
 
-        @self._player.on_key_press('u')
+        @keypress(settings.kb_unwatched)
         def handle_unwatched():
             self.put_task(self.unwatched_quit)
 
-        @self._player.on_key_press('c')
+        @keypress(settings.kb_menu)
         def menu_open():
             if not self.menu.is_menu_shown:
                 self.menu.show_menu()
             else:
                 self.menu.hide_menu()
         
-        @self._player.on_key_press('esc')
+        @keypress(settings.kb_menu_esc)
         def menu_back():
-            self.menu.menu_action('back')
+            if self.menu.is_menu_shown:
+                self.menu.menu_action('back')
+            else:
+                self._player.command('set', 'fullscreen', 'no')
 
-        @self._player.on_key_press('enter')
+        @keypress(settings.kb_menu_ok)
         def menu_ok():
             self.menu.menu_action('ok')
         
-        @self._player.on_key_press('left')
+        @keypress(settings.kb_menu_left)
         def menu_left():
             if self.menu.is_menu_shown:
                 self.menu.menu_action('left')
             else:
-                seektime = -5
+                seektime = settings.seek_left
                 if settings.use_web_seek:
                     seektime, _ = self.get_seek_times()
                 self._player.command("seek", seektime)
         
-        @self._player.on_key_press('right')
+        @keypress(settings.kb_menu_right)
         def menu_right():
             if self.menu.is_menu_shown:
                 self.menu.menu_action('right')
             else:
-                seektime = 5
+                seektime = settings.seek_right
                 if settings.use_web_seek:
                     _, seektime = self.get_seek_times()
                 self._player.command("seek", seektime)
 
-        @self._player.on_key_press('up')
+        @keypress(settings.kb_menu_up)
         def menu_up():
             if self.menu.is_menu_shown:
                 self.menu.menu_action('up')
             else:
-                self._player.command("seek", 60)
+                self._player.command("seek", settings.seek_up)
 
-        @self._player.on_key_press('down')
+        @keypress(settings.kb_menu_down)
         def menu_down():
             if self.menu.is_menu_shown:
                 self.menu.menu_action('down')
             else:
-                self._player.command("seek", -60)
+                self._player.command("seek", settings.seek_down)
 
-        @self._player.on_key_press('space')
+        @keypress(settings.kb_pause)
         def handle_pause():
             if self.menu.is_menu_shown:
                 self.menu.menu_action('ok')
@@ -205,7 +217,7 @@ class PlayerManager(object):
                 self.toggle_pause()
 
         # This gives you an interactive python debugger prompt.
-        @self._player.on_key_press('~')
+        @keypress(settings.kb_debug)
         def handle_debug():
             import pdb
             pdb.set_trace()
@@ -214,13 +226,15 @@ class PlayerManager(object):
         @self._player.property_observer('eof-reached')
         def handle_end(_name, reached_end):
             if self._video and reached_end:
-                self.put_task(self.finished_callback)
+                has_lock = self._finished_lock.acquire(False)
+                self.put_task(self.finished_callback, has_lock)
 
         # Fires at the end.
         @self._player.event_callback('idle')
         def handle_end_idle(event):
             if self._video:
-                self.put_task(self.finished_callback)
+                has_lock = self._finished_lock.acquire(False)
+                self.put_task(self.finished_callback, has_lock)
 
     # Put a task to the event queue.
     # This ensures the task executes outside
@@ -288,6 +302,8 @@ class PlayerManager(object):
         self.send_timeline_initial()
         self._player.pause = False
         self.should_send_timeline = True
+        if self._finished_lock.locked():
+            self._finished_lock.release()
 
     def exec_stop_cmd(self):
         if settings.stop_cmd:
@@ -366,14 +382,17 @@ class PlayerManager(object):
         return False
 
     @synchronous('_lock')
-    def finished_callback(self):
+    def finished_callback(self, has_lock):
         if not self._video:
             return
        
         self._video.set_played()
         if self._video.parent.has_next and settings.auto_play:
-            log.debug("PlayerManager::finished_callback starting next episode")
-            self.play(self._video.parent.get_next().video)
+            if has_lock:
+                log.debug("PlayerManager::finished_callback starting next episode")
+                self.play(self._video.parent.get_next().video)
+            else:
+                log.debug("PlayerManager::finished_callback No lock, skipping...")
         else:
             if settings.media_ended_cmd:
                 os.system(settings.media_ended_cmd)
